@@ -2,7 +2,6 @@
 
 #[cfg(not(portable_atomic_no_asm))]
 use core::arch::asm;
-use core::sync::atomic::{compiler_fence, Ordering};
 
 #[inline]
 pub(super) fn is_enabled() -> bool {
@@ -24,18 +23,10 @@ pub(super) fn disable() {
         #[cfg(portable_atomic_no_asm)]
         llvm_asm!("cli" :::: "volatile");
     }
-
-    // Ensure no subsequent memory accesses are reordered to before interrupts are disabled.
-    // https://github.com/rust-embedded/cortex-m/pull/264
-    compiler_fence(Ordering::SeqCst);
 }
 
 #[inline]
 pub(super) unsafe fn enable() {
-    // Ensure no preceding memory accesses are reordered to after interrupts are enabled.
-    // https://github.com/rust-embedded/cortex-m/pull/264
-    compiler_fence(Ordering::SeqCst);
-
     unsafe {
         #[cfg(not(portable_atomic_no_asm))]
         asm!("sei", options(nomem, nostack));
